@@ -2,3 +2,5 @@ mod test_files;
 mod test_hydrus;
 mod test_import;
 mod test_url;
+mod test_page;
+mod test_address;
